<!--- 
	csw-videotextblock-playeronpage.cfc
	
	This is the core rendering for the 'player on page' container renderhandler for the videotextblock widget.
	
	Version: [2018-01-18 16:45]
--->

<cfcomponent displayname="csw-videotextblock-playeronpage" output="false" hint="Video Textblock Player On Page Functions" extends="csw-videotextblock">


	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			// load resources from base icon textblock component
			super.loadResources();
			
			Server.CommonSpot.udf.resources.loadResources("jQueryActualSize");
		</cfscript>
	</cffunction>
	


	<cffunction name="loadProperties" access="public" returntype="struct" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW VideoTextBlock Player On Page";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;;
			var widgetData = StructNew();

			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;	
				
			// 
			// Get Metadata (Core Image Text	block metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];
			
			
			/***************************************************/
			/****** VIDEO PLAYER CONTAINER PROPERTIES TAB ******/
			/***************************************************/
			
			// Player Container ID
			props.PlayerContainerID = getStandardField( widgetData=widgetData, field='Player_Container_Id', defValue='cswVideoPlayerOnPage-#request.element.id#' );
			
			// Player Width
			props.PlayerWidth = getStandardField( widgetData=widgetData, field='Player_Width', defValue='100' );
		
			// Player Position Fixed
			props.PlayerPositionFixed = getStandardField( widgetData=widgetData, field='Player_Position_Fixed', defValue='cswVideoPlayer-relative' );
			
			// Fixed Position Top
			props.PlayerFixedTop = getStandardField( widgetData=widgetData, field='Player_Fixed_Top', defValue='150' );

			// Disable Fixed Position Breakpoint
			props.DisableFixedBreakPoint = getStandardField( widgetData=widgetData, field='Disable_Fixed_BreakPoint', defValue='480' );
			
			return props;
		</cfscript>
	</cffunction>


	<!----------------------------------------
		Render Video Player Container
	----------------------------------------->
	<cffunction name="renderVideoPlayerContainer" access="public" output="Yes" returntype="void">
		<cfargument name="elementInfo" required="Yes" type="struct">
		<cfargument name="propsStruct" required="Yes" type="struct">
		<cfargument name="elementID" type="string" required="Yes">

		<cfscript>
			var data = StructNew();
			var props = arguments.propsStruct;
			var containerID = props.PlayerContainerID;
			var playerID = 'vid_' & containerID;
			var playerStyleAttr = '';
			var PlayerFixedTop = 0;
			var disableFixedBreakpoint = 480;
			var videoMsg = '';
			//var videoMsg = 'Render Video Here! <br/> Use ID: #containerID#';
			var videoErrorMsg = '';
			// Set a container ID for the video list
			var listingContainerID = "vidList_" & ReplaceNoCase(containerID,'vid_','','one');
			
			if ( IsNumeric(props.PlayerWidth) ) 
				playerStyleAttr = playerStyleAttr &  ' width: #props.PlayerWidth#%;';
				
			if ( LEN(TRIM(playerStyleAttr)) ) 
				playerStyleAttr = 'style="#playerStyleAttr#"';
				
			if ( IsNumeric(props.PlayerFixedTop) )
				PlayerFixedTop = props.PlayerFixedTop;

			if ( StructKeyExists(props,'DisableFixedBreakPoint') AND IsNumeric(props.DisableFixedBreakPoint) )
				disableFixedBreakpoint = props.DisableFixedBreakPoint;

			// Set elementInfo back to attributes for the container schedule-standard-render.cfm
			attributes.ElementInfo = arguments.elementinfo;
		</cfscript>
		
		<cfsavecontent variable="playerOnPageFooterJS">
		<cfoutput>
			jQuery(document).ready(function() {	
			
				// Init the resize() event
				jQuery(window).resize();

			});

			jQuery(window).resize(function() {
				
				// Set the Player Container Height based on the width
				setPlayerOnPageContainerHeight('#containerID#','#Val(props.PlayerWidth)#%');
				
				<cfif props.PlayerPositionFixed EQ "cswVideoPlayer-fixed">
				
				// Set the Player Fixed Postion if Enabled (disable for mobile)
				setPlayerOnPageFixedPosition('#playerID#','#listingContainerID#',#Val(PlayerFixedTop)#,#Val(disableFixedBreakpoint)#);
				
				</cfif>
				
			});
		</cfoutput>
		</cfsavecontent>
		
		<cfscript>
			Server.CommonSpot.udf.resources.addFooterJS(playerOnPageFooterJS, 'TERTIARY');
		</cfscript>
		
		<cfoutput>
			<div id="#playerID#" class="cswVideoPlayerOnPage cswPlayerIcon cswIconFirstLoad" #playerStyleAttr#>
				<!--- #videoMsg# --->
			</div>
		</cfoutput>
		
		<cfinclude template="/commonspot/controls/schedule/schedule-standard-render.cfm">

	</cffunction>

</cfcomponent>