<!--- 
	csw-videotextblock.cfc
	
	This is the core rendering for the image-textblock-videolist widget.

	Extends: /csw-imagetextblock/renderhandlers/csw-imagetextblock.cfc

	Version: [2018-01-16 15:20]
--->


<cfcomponent displayname="csw-videotextblock" output="false" hint="Common Video Textblock Functions" extends="_cs_widgets.csw-imagetextblock.renderhandlers.csw-imagetextblock"> 

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			// load resources from base icon textblock component
			super.loadResources();

			Server.CommonSpot.udf.resources.loadResources("Bootstrap");
			Server.CommonSpot.udf.resources.loadResources("cswVideoTextblock");

			// If a custom stylesheets have been registered, go ahead and load them.
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswVideoTextblock_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswVideoTextblock_custom");
		</cfscript>
	</cffunction>
	
	
	<cffunction name="loadProperties" access="public" returntype="struct" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW ImageTextblock";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;;
			var widgetData = StructNew();

			// load properties from base icon textblock
			props = super.loadProperties(arguments.elementInfo,arguments.rhCaller,false);
			
			
			// These are for the icon-counter-textblock widget.
			// If they don't exist we won't render them out.
			widgetMetadataFormName = "CSW VideoTextBlock";
			
			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;	
				
			// 
			// Get Metadata (Core Image Text	block metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];
				
			props.video = StructNew();
			
			/**********************************/
			/****** VIDEO PROPERTIES TAB ******/
			/**********************************/
				
			// Video Style 
			props.video.Style = getStandardField( widgetData=widgetData, field='Video_Style', defValue='cswVideo-lightbox' );
		
			// Video Player Container Id 
			props.video.PlayerContainerID = getStandardField( widgetData=widgetData, field='Video_Player_Container_Id', defValue='cswVideo-lightbox' );

			// Video Lightbox Width
			props.video.LightboxWidth = getStandardField( widgetData=widgetData, field='Video_Lightbox_Width', defValue='65' );
		
			// AutoPlay  
			props.video.AutoPlay = getStandardField( widgetData=widgetData, field='Auto_Play', defValue='0' );

			// Video Loop 
			props.video.Loop = getStandardField( widgetData=widgetData, field='Loop', defValue='0' );
				
			// Video Default Button Text
			props.video.DefaultButtonText = getStandardField( widgetData=widgetData, field='Default_Button_Text', defValue='Click' );
			
			// Video Icon 
			props.videoIcon = getStandardField( widgetData=widgetData, field='Video_Icon', default='cswVideoThumbnail-show' );
			
			// Video Icon (size & color)
			props.videoIconSettings = getFieldSettings( widgetData=widgetData, field='Video_Icon_Settings' );


			return props;
		</cfscript>
	</cffunction>


	<!----------------------------------------
		Get Items (Override to get PageIndex Data for the RH, instead of Custom Element Data) 
	----------------------------------------->
	<cffunction name="getItems" access="public" output="Yes" returntype="array">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		
		<cfscript>
			var elementData = arguments.elementDataStruct;
			var elementID = request.element.ID;
			var items = ArrayNew(1);
			var videoTypeSkipList = "Local"; // Local,Brigetcove,YouTube
			var i = 1;
			var data = ArrayNew(1);
			var filteredData = ArrayNew(1);
			var hasVideoItems = false;
			var errMsg = "";
			
			
			// Get Records from the Generic Render Handler propertyValues Array of Structures
			if ( StructKeyExists(elementData, 'propertyValues') AND ArrayLen(elementData.propertyValues) )
				data = elementData.propertyValues;
			// Get Records from the Page Index Element as an Array of Structures
			else if ( StructKeyExists(elementData, 'items') AND ArrayLen(elementData.items) )
				data = elementData.items;
				
				
			// Filter out items in the videoType skiplist
			for( i=1; i lte ArrayLen(data); i=i+1 )
			{
				if ( StructKeyExists(data[i],"values") ) 
					items[i] = data[i].values;	
				else
					items[i] = data[i];
					
				// Get Video Info (title, length, etc)
				items[i].videoProps = getVideoProps(items[i].id);
//	writedump(items[i].videoProps);

				// Only render Video PageIndex Items that are not in the videoType SKIP list
				if ( StructKeyExists(items[i].videoProps,"ServiceName") AND ListFindNoCase(videoTypeSkipList,items[i].videoProps.ServiceName) EQ 0  )
					arrayAppend(filteredData,items[i]);
			}
			
			
			// Check, after filtering, if we have any video items to render
			if ( ArrayLen(filteredData) )
				hasVideoItems = true;
			else
			{
				errMsg = "No videos available.";
				if ( request.user.id gt 0 AND server.commonspot.udf.security.authorok() == 1 )
					errMsg = errMsg & " Please configure external Multimedia files. Local Multimedia files are not compatible.";

				WriteOutput('<div><em>#errMsg#</em></div>');
				
				return ArrayNew(1);
			}

			// Pass the hasVideoItems to the player render as a request variable
			request.cswHasVideoItems[elementID] = hasVideoItems;

			return filteredData;
		</cfscript>

	</cffunction>
	
	<!----------------------------------------
		Get And Render Items
	----------------------------------------->
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="mode" type="string" required="false" default="all">
		
		<cfscript>
			var props = arguments.propsStruct;
			var elementID = request.element.ID;
			var videoIconClasses = '';
			
			// set the container ID for the video player
			props.containerID = elementID;
			
			if ( props.video.style EQ "cswVideo-onpage" )
			{
				if ( StructKeyExists(props.video,"PlayerContainerID") )
					props.containerID = props.video.PlayerContainerID;
			}
			
			// Video Icon				
			if( props.VideoIcon neq 'cswVideoIcon-none' )			
			{
				videoIconClasses = props.VideoIcon & ' ' & props.videoIconSettings.size;
			}
							
			// Set a container ID for the video list
			props.listingContainerID = "vidList_" & ReplaceNoCase(props.containerID,'vid_','','one');
		</cfscript>
	
		<cfoutput>
			<div id="#props.listingContainerID#" class="cswVideoTextblock #videoIconClasses#">
				#super.getAndRenderItems(elementDataStruct=arguments.elementDataStruct,
													propsStruct=props,
													mode=arguments.mode)#
			</div>
			<cfif props.VideoIcon neq 'cswVideoIcon-none'>			
				<style>
					##vidList_#request.element.id#.cswVideoIcon-show .cswVideoThumbnail:after, 
					##vidList_#request.element.id#.cswVideoIcon-hideOnHover .cswVideoThumbnail:after,
					##vidList_#request.element.id#.cswVideoIcon-showOnHover .cswImageWrapper > a:hover > .cswVideoThumbnail:after {
						color: #props.videoIconSettings.color#;
					}			
				</style>
			</cfif>
		</cfoutput>

	</cffunction>

	<!----------------------------------------
		Render Items
	----------------------------------------->
	<cffunction name="RenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="itemsArray" type="array" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="mode" type="string" required="Yes">

		<cfscript>
			var i = 1;
			var start = 1;
			var totalItems = 1;
			var items = arguments.itemsArray;
			var props = arguments.propsStruct;
			var data = StructNew();
			var elementID = request.element.ID;
			var containerID = '';

			data.itemid = '';
			data.name = '';
			data.title = '';
			data.docType = '';
			data.desc = '';
			data.image = '';
			data.subtitle = '';
			data.imageFocalPoint = 'cswImgPos-centermiddle'; // Hard position
			data.linkText = 'View';
			data.linkURL = '';
			data.video = StructNew();
			data.videoID = '';
			data.onClick = '';
			
			// Set Default if value not passed in
			if ( !StructKeyExists(props.video,"autoplay") )
				props.video.autoplay = 0;
			if ( !StructKeyExists(props.video,"loop") )
				props.video.loop = 0;
			
			// Set Default linkText if value exists
			if ( StructKeyExists(props.video,"DefaultButtonText") )
				data.linkText = props.video.DefaultButtonText;
							
			// Determine Mode
			if( arguments.mode eq 'first' )
			{
				start = 1;
				totalItems = 1;
			}
			else if( arguments.mode eq 'rest' )
			{
				start = 2;
				totalItems = ArrayLen(items);
			}
			else
			{
				start = 1;
				totalItems = ArrayLen(items);
			}


			// Loop appropriately
			for( i=start; i lte totalItems; i=i+1 )
			{
				// cs item id
				data.itemid = items[i].id;

				// Name
				data.name = items[i].name;

				// Title (Required)
				data.title = items[i].title;

				// docType
				data.docType = items[i].docType;
				
				// Get Video Info (title, length, etc)
				data.video = items[i].videoProps;
				
				// Get the External Video ID
				data.videoID = '';
				if ( StructKeyExists(data.video,"extReferenceID") )
					data.videoID = data.video.extReferenceID;

				// Set default Image value
				data.image = '';
				// Look for image in mapped Image Field
				if( StructKeyExists(items[i],'Image') )
					data.image = getStandardCustomElementImageField(widgetData=items[i].image,field='Image',defValue='');

				// If No data.image was found, then use the Video Thumbnail
				if( Len(Trim(data.image)) EQ 0 AND isStruct(data.video) AND StructKeyExists(data.video,'ThumbnailImageURL') )
				{
					// get image URL from the Multimedia Element data
					if( Len(Trim(data.video.ThumbnailImageURL)) )
						data.image = data.video.ThumbnailImageURL;
				}


				// Subtitle (Optional)
				if( StructKeyExists(items[i],'Subtitle') )
					data.subtitle = items[i].Subtitle;

				if( StructKeyExists(items[i],'Description') )
					data.desc = items[i].Description;


				// Build JS call properties
				data.LinkURL = '';
				data.onClick = '';
				data.linkAttrStruct = StructNew();
				if ( data.video.title neq "")
				{
					
					// Check the video player style (cswVideo-onpage OR cswVideo-lightbox)
					if ( props.video.style EQ "cswVideo-onpage" )
					{
						data.LinkURL = "##";
						
						//props.containerID = elementID;
						//if ( StructKeyExists(props.video,"PlayerContainerID") )
						//	props.containerID = props.video.PlayerContainerID;
						// Set a container ID for the video list
						//props.listingContainerID = "vidList_" & ReplaceNoCase(props.containerID,'vid_','','one');
							
						// Add the Pre-Click video container JS to the onClick String
						data.onClick = data.onClick & renderVideoContainerPreClickJS(containerID=props.containerID);
						
						// Build the JS for the Anchor Tag onClick
						data.onClick = data.onClick & renderVideoPlayerLoaderJS(containerID=props.containerID,
																									itemID=data.itemid,
																									videoID=data.videoID,
																									title=data.title,
																									docType=data.docType,
																									autoplay=props.video.autoplay,
																									loop=props.video.loop);
					
						// Add the Post-Click video container JS to the onClick String
						data.onClick = data.onClick & renderVideoContainerPostClickJS(containerID=props.containerID);
					}
					else
					{
						data.LinkURL = "##videoPlayerDlg" & elementID;
						
						// Add the Pre-Click video modal player JS to the onClick String
						data.onClick = data.onClick & renderVideoModalPreClickJS(containerID=elementID,
																										modalID=data.LinkURL);
						
						// Build the JS for the Anchor Tag onClick
						data.onClick = data.onClick & renderVideoPlayerLoaderJS(containerID=elementID,
																									itemID=data.itemid,
																									videoID=data.videoID,
																									title=data.title,
																									docType=data.docType,
																									autoplay=props.video.autoplay,
																									loop=props.video.loop);
						
						// Add the Post-Click video modal player JS to the onClick String
						data.onClick = data.onClick & renderVideoModalPostClickJS(containerID=elementID,
																										modalID=data.LinkURL,
																										title=data.title,
																										desc='');
						
					
						data.linkAttrStruct = StructNew();
						data.linkAttrStruct['data-toggle'] = 'modal';
						data.linkAttrStruct['data-modaltitle'] = '#JSStringFormat(data.title)#';
					}
					
					// Close the OnClick string
					data.onClick = data.onClick & "return false;";
					
				}

				// RENDER the Image Textblock container
				renderImageTextBlock( data, props, i );

			}
		</cfscript>

	</cffunction>


	<!----------------------------------------
		Render Title (Override for the Video Title and Video Link)
	----------------------------------------->
	<cffunction name="renderTitle" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
			var bgColor = '';
			
			if( StructKeyExists(props.title,'show') AND props.title.show neq 'no' )
			{
				renderTextCore(
					class = 'cswTitle cswVideoTitle',
					tag = props.title.tag,
					text = data.title,
					color = props.title.color,
					backgroundColor = props.title.bgColor,
					size = props.title.size,
					align = props.title.align,
					fontClasses = props.title.fontClasses,
					padding = props.title.padding,
					effect = props.title.effect,
					linkURL = data.linkURL,
					style = '',
					onClick = data.onClick,
					linkAttrStruct = data.linkAttrStruct );
			}
		</cfscript>
	</cffunction>
	
	
	<!----------------------------------------
		Render Image
	----------------------------------------->
	<cffunction name="renderImage" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="imgWrapperStyle" type="string" required="No" default="">
		<cfargument name="imgWrapperClass" type="string" required="No" default="">
		
		<cfscript>
			var linkClasses = 'cswVideoImageLink';
			
			if ( NOT StructKeyExists(propsStruct,"imageHeightType") ) 
				propsStruct.imageHeightType = '';
				
			if ( NOT StructKeyExists(dataStruct,"imageFocalPoint") ) 
				dataStruct.imageFocalPoint = '';
				
			// Link Classes
			if( !StructKeyExists(propsStruct,'linkClass') )
				propsStruct.linkClass = "#linkClasses#";
			else
				propsStruct.linkClass = "#propsStruct.linkClass# #linkClasses#";
					
			if ( FindNoCase('auto', propsStruct.imageHeightType) )
				propsStruct.linkClass = propsStruct.linkClass & " cswVideoThumbnail";
			else 
				dataStruct.imageFocalPoint	= dataStruct.imageFocalPoint & " cswVideoThumbnail";
		</cfscript>
		
		<cfoutput>#super.renderImage(argumentcollection=arguments)#</cfoutput>
		
	</cffunction>
	
	
	<!----------------------------------------
		Verify Has Video Items 
	----------------------------------------->
	<cffunction name="verifyHasVideoItems" access="public" output="Yes" returntype="boolean">
		<cfargument name="elementID" type="string" required="Yes">
		
		<cfscript>
			if ( NOT StructKeyExists(request,"cswHasVideoItems") 
						OR NOT StructKeyExists(request.cswHasVideoItems, arguments.elementID) 
						OR request.cswHasVideoItems[arguments.elementID] EQ false )
				return false;
				
			return true;
		</cfscript>
	</cffunction>
	
	<!----------------------------------------
		Render Video Player
	----------------------------------------->
	<cffunction name="renderVideoPlayer" access="public" output="Yes" returntype="void">
		<cfargument name="elementID" type="string" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var props = arguments.propsStruct;
		 	var modalID = "videoPlayerDlg#arguments.elementID#";
			var playerID = "vid_#arguments.elementID#"; 
			var containerID = arguments.elementID; // Use the elementid as the containerID default
			var useVideoModalDialog = false;
			var lightboxWidth = 80;
			
			// Check we have video item records
			if ( verifyHasVideoItems(elementID=arguments.elementID) )
			{			
				// Check the video player style (cswVideo-onpage OR cswVideo-lightbox)
				if ( props.video.style EQ "cswVideo-onpage" )
				{
					if ( StructKeyExists(props.video,"PlayerContainerID") )
						containerID = props.video.PlayerContainerID;

					renderVideoPlayerOnPageContainerJS(containerID=props.containerID);
				}
				else
				{
					// Set to use Modal Dialog for video player
					useVideoModalDialog = true;

					if ( StructKeyExists(props.video,"LightboxWidth") AND IsNumeric(props.video.LightboxWidth) )
						lightboxWidth = props.video.LightboxWidth;

					// Render one Video Player Modal Dialog
					renderVideoPlayerModalDialog(modalId=modalID,playerID=playerID,modalWidth=lightboxWidth);
				}
			}
		</cfscript>
	</cffunction>
	
	
	<cffunction name="renderVideoPlayerOnPageContainerJS" access="public" output="Yes" returntype="void">
		<cfargument name="containerID" required="true" type="string">
		
		<cfscript>
			var playerOnPageContainerFooterJS = '';
		</cfscript>
			
		<cfsavecontent variable="playerOnPageContainerFooterJS">
		<cfoutput>
			jQuery(document).ready(function() {	
				clearPlayerOnPageContainer('#arguments.containerID#');
			});
		</cfoutput>
		</cfsavecontent>
		
		<cfscript>
			Server.CommonSpot.udf.resources.addFooterJS(playerOnPageContainerFooterJS, 'TERTIARY');
		</cfscript>
	</cffunction>
	

	<!----------------------------------------
		Get Video Player Modal Dialog
	----------------------------------------->
	<cffunction name="renderVideoPlayerModalDialog" access="public" output="Yes" returntype="void">
		<cfargument name="modalId" required="false" type="string" default="videoPlayerDlg">
		<cfargument name="modalClass" required="false" type="string" default="fade videoPlayerModal">
		<cfargument name="modalTitle" required="false" type="string" default="Video Player">
		<cfargument name="modalWidth" required="false" type="string" default="">
		<cfargument name="playerID" required="false" type="string" default="videoPlayer">
		<cfargument name="videoErrorMsg" required="false" type="string" default="Oh no, something went wrong with video you selected!">
		<cfargument name="videoDefaultDesc" required="false" type="string" default="">

		<cfscript>
			var modalStyleAttr = "";
			var modalBodyStyleAttr = "";
			var modalContentStyleAttr = "";
			var playerStyleAttr = "";
			var playerModalFooterJS = "";

			if ( Len(arguments.modalWidth) )
			{
				modalStyleAttr = modalStyleAttr & "width:#arguments.modalWidth#%;";
				modalStyleAttr = modalStyleAttr & "margin-left:auto;margin-right:auto;";
				
				modalContentStyleAttr = modalContentStyleAttr & "min-height:100%;height:auto;";

				if ( Len(modalStyleAttr) )
					modalStyleAttr = 'style="#modalStyleAttr#"';
					
				if ( Len(modalContentStyleAttr) )
					modalContentStyleAttr = 'style="#modalContentStyleAttr#"';
					
				if ( Len(modalBodyStyleAttr) )
					modalBodyStyleAttr = 'style="#modalBodyStyleAttr#"';
			}
			
			if ( NOT StructKeyExists(request,"VideoPlayerModalDialog") )
				request.VideoPlayerModalDialog = StructNew();
		</cfscript>
		
		<cfif NOT StructKeyExists(request.VideoPlayerModalDialog,arguments.modalId)>
			<cfoutput>
				<div class="modal #arguments.modalClass#" tabindex="-1" role="dialog" id="#arguments.modalId#" aria-labelledby="#arguments.modalTitle#" aria-hidden="true">
					<div class="modal-dialog" role="document" #modalStyleAttr#>
						<div class="modal-content" #modalContentStyleAttr#>
						  <div class="modal-header">
							 <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
							 <h3 class="modal-title">#arguments.modalTitle#</h3>
						  </div>
						  <div class="modal-body" #modalBodyStyleAttr#>

								<div id="#arguments.playerID#" class="cswVideoPlayer" #playerStyleAttr#>#arguments.videoErrorMsg#</div>
								<div id="#arguments.playerID#_desc" class="cswVideoDesc" #playerStyleAttr#>#arguments.videoDefaultDesc#</div>

							</div> <!-- /.model-body -->
							<div class="modal-footer">
								<button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
							</div>
						</div><!-- /.modal-content -->
					</div><!-- /.modal-dialog -->
				</div><!-- /.modal -->
			</cfoutput>
		
			<!--- // Clean up when modal widow closes --->
			<cfsavecontent variable="playerModalFooterJS">
			<cfoutput>
				jQuery(document).ready(function() {	
					removeVideoAndTextFromModalOnClose('#arguments.modalId#');
				});
			</cfoutput>
			</cfsavecontent>
		
			<cfscript>
				Server.CommonSpot.udf.resources.addFooterJS(playerModalFooterJS, 'TERTIARY');
			</cfscript>
			
			<cfset request.VideoPlayerModalDialog[arguments.modalId] = 1>
		</cfif>
	</cffunction>

	
	<!----------------------------------------
		Get Video Props
	----------------------------------------->
	<cffunction name="getVideoProps" access="public" output="Yes" returntype="struct">
		<cfargument name="itemid" type="numeric" required="Yes">

		<cfscript>
			var retStruct = StructNew();
			var mmObj = Server.commonspot.ObjectFactory.getObject('Multimedia');

			// Get Video Info (title, length, etc)
			retStruct = mmObj.getInfo(arguments.itemid);

			return retStruct;
		</cfscript>
	</cffunction>
	
	<!----------------------------------------
		Render Video Modal Pre-Click JS
	----------------------------------------->	
	<cffunction name="renderVideoModalPreClickJS" access="public" output="Yes" returntype="string">
		<cfargument name="containerID" type="string" required="yes">
		<cfargument name="modalID" type="string" required="yes">

		<cfscript>
			return "";
		</cfscript>
	</cffunction>

	<!----------------------------------------
		Render Video Modal Post-Click JS
	----------------------------------------->	
	<cffunction name="renderVideoModalPostClickJS" access="public" output="Yes" returntype="string">
		<cfargument name="containerID" type="string" required="yes">
		<cfargument name="modalID" type="string" required="yes">
		<cfargument name="title" type="string" required="yes">
		<cfargument name="description" type="string" required="no" default="">

		<cfscript>
			// Return the Modal Title and Description Updater JS
			return "updateVideoModalText('#arguments.modalID#','#JSStringFormat(arguments.title)#','#JSStringFormat(arguments.description)#');";
		</cfscript>
	</cffunction>
	
	
	<!----------------------------------------
		Render Video Container Pre-Click JS
	----------------------------------------->	
	<cffunction name="renderVideoContainerPreClickJS" access="public" output="Yes" returntype="string">
		<cfargument name="containerID" type="string" required="yes">

		<cfscript>
			// Return the Hide Player PlaceHolder JS
			return "hidePlayerOnPagePlaceHolder('#arguments.containerID#');";
		</cfscript>
	</cffunction>
	
	
	<!----------------------------------------
		Render Video Container Post-Click JS
	----------------------------------------->	
	<cffunction name="renderVideoContainerPostClickJS" access="public" output="Yes" returntype="string">
		<cfargument name="containerID" type="string" required="yes">

		<cfscript>
			return "";
		</cfscript>
	</cffunction>


	<!----------------------------------------
		Render Video Player Loader JS
	----------------------------------------->
	<cffunction name="renderVideoPlayerLoaderJS" access="public" output="Yes" returntype="string">
		<cfargument name="containerID" type="string" required="yes" >
		<cfargument name="itemID" type="string"  required="yes">
		<cfargument name="videoID" type="string" required="yes">
		<cfargument name="title" type="string" required="yes">
		<cfargument name="docType" type="string" required="yes">
		<cfargument name="autoplay" type="boolean" required="false" default="1">
		<cfargument name="loop" type="boolean" required="false" default="1">

		<cfscript>
			var videoProps = getVideoPlayerProps(arguments.itemid,arguments.title,arguments.docType);
			var jsArgs = getVideoPlayerJsArguments(arguments.videoID, videoProps.playerRender, videoProps.player.playerID, arguments.autoplay, arguments.loop);

			return "#jsArgs.fname#('#arguments.containerID#', #jsArgs.obj#, aList='#jsArgs.argList#');"; //return false;
		</cfscript>
	</cffunction>


	<!----------------------------------------
		Render Video Player Props
	----------------------------------------->
	<cffunction name="getVideoPlayerProps" access="public" output="Yes" returntype="struct">
		<cfargument name="itemID" type="string"  required="yes">
		<cfargument name="title" type="string" required="yes">
		<cfargument name="docType" type="string" required="yes">

		<cfscript>
			var retStruct = StructNew();
			var playerObj = Server.CommonSpot.ObjectFactory.getObject("MultimediaPlayer");

			// Get default Player info
			retStruct.player = playerObj.getDefaultPlayer(multimediaObjectID=arguments.itemid,includePlayerParamStruct=1);

			// get Player Attributes
			if (IsStruct(retStruct.player) AND StructKeyExists(retStruct.player, "playerID") AND val(retStruct.player.PlayerID) gt 0)
				retStruct.playerRender = playerObj.getRenderDefaults(playerID=retStruct.player.playerID, playerData=retStruct.player.playerData);
			else
			{
				writeOutput('<div>#arguments.Title# (No default player found for file type #arguments.docType#.)</div>');
				continue;
			}

			return retStruct;
		</cfscript>
	</cffunction>


	<!----------------------------------------
		Get Video Player JS Arguments
	----------------------------------------->
	<cffunction name="getVideoPlayerJsArguments" returnType="struct" returnFormat="json">
		<cfargument name="videoID" required="yes" type="string">
		<cfargument name="playerRender" type="struct" required="yes">
		<cfargument name="playerID" type="any" required="yes">
		<cfargument name="autoplay" type="string" required="false" default="" hint="Empty String: Use Player default autoplay setting">
		<cfargument name="loop" type="string" required="false" default="" hint="Empty String: Use Player default loop setting">
		<cfargument name="width" type="string" required="false" default="" hint="Empty String: Use Player default Width setting, None: Removes Width attribute">
		<cfargument name="height" type="string" required="false" default="" hint="Empty String: Use Player default Heigth setting, None: Removes Heigth attribute">

		<cfscript>
			var attribArr = [];
			var linkArgs = {};
			var kName = "";
			var kVal = "";
			var i = 0;
			var fn = "";
			var replaceString = "player_";
			var args = "videoID='#videoID#'";
			var argList = "videoID";
			var additionalInfo = "";
			var getPlayerInfo = 0;
			var pInfo = "";
			var cObj = "";
			var rInfo = arguments.playerRender;
			var protocol = "http:";
			var host = cgi.http_host;

			if ( Request.CGIVars.CS_Secure_Request )
				protocol = "https:";

			if (rInfo.serviceID == Request.Constants.factoryMultimediaService.YOUTUBE_ID)
			{
				attribArr = ListToArray("height,autohide,autoplay,color,controls,end,fs,loop,modestbranding,origin,playerapiid,showinfo,start,theme,width");
				fn = "renderYTVideo";
				replaceString = "player_playerVars_";
			}
			else if (rInfo.serviceID == Request.Constants.factoryMultimediaService.BRIGHTCOVE_ID)
			{
				attribArr = ListToArray("autoplay,controls,height,loop,muted,preload,width");
				fn = "renderBCVideo";
				getPlayerInfo = 1;
			}
			else
			{
				attribArr = ListToArray("clsid,height,mimetype,src,width,src_mp4_url,src_ogv_url,src_webm_url,autoplay,loop");
				fn = "renderLocalVideo";
			}
		</cfscript>

		<cfif getPlayerInfo>
			<cfquery name="playerInfo" datasource="#request.site.datasource#">
				SELECT mp.SettingsXML, mc.properties
				  FROM multimediaPlayers mp, multimediaChannels mc
				 WHERE mp.serviceID = <cfqueryparam cfsqltype="cf_sql_integer" value="#rInfo.serviceID#">
					AND mp.channelID = mc.ID
					AND mc.serviceID = <cfqueryparam cfsqltype="cf_sql_integer" value="#rInfo.serviceID#">
					AND mp.ID = <cfqueryparam cfsqltype="cf_sql_integer" value="#arguments.playerID#">
			</cfquery>

			<cfscript>
				if (playerInfo.properties neq "")
				{
					pInfo = Server.CommonSpot.UDF.util.deserialize(playerInfo.properties);
					args = "#args#, publisherID='#pInfo.publisherid#'";
					argList = "#argList#,publisherID";
				}
				if (playerInfo.SettingsXML neq "")
				{
					pInfo = Server.CommonSpot.UDF.util.deserialize(playerInfo.SettingsXML);
					for (i=1; i le arrayLen(pInfo); i=i+1)
					{
						cObj = pInfo[i];
						if (Isstruct(cObj) AND StructKeyExists(cObj,"name") AND cObj['name'] eq 'data-player')
						{
							args = "#args#, dataPlayer='#cObj.value#'";
							argList = "#argList#,dataPlayer";
						}
					}
				}
			</cfscript>
		</cfif>

		<cfscript>
			for (key in rInfo)
			{
				kName = ReplaceNoCase(key, replaceString, "");
				kVal = rInfo[key];
				if (arrayFind(attribArr, kName) gt 0 && kVal != '' && FindNoCase("CP___PAGEID=", kVal) eq 0)
				{
					addParam = true;

					// Overrider the AUTOPLAY setting from player
					if ( kName EQ 'autoplay' )
					{
						if ( IsNumeric(arguments.autoplay) AND ( arguments.autoplay GTE 0 OR arguments.autoplay LTE 1 ) )
							kVal = arguments.autoplay;
					}

					// Overrider the LOOP setting from player
					if ( kName EQ 'loop' )
					{
						if ( IsNumeric(arguments.loop) AND ( arguments.loop GTE 0 OR arguments.loop LTE 1 ) )
							kVal = arguments.loop;
					}

					// Overrider the width setting from player
					if ( kName EQ 'width' )
					{
						if ( Len(arguments.width) )
						{
							if ( arguments.width EQ 'none' )
								kVal = arguments.width;
							else
								addParam = false;
						}
					}

					// Overrider the height setting from player
					if ( kName EQ 'height' )
					{
						if ( Len(arguments.height) )
						{
							if ( arguments.height EQ 'none' )
								kVal = arguments.height;
							else
								addParam = false;
						}
					}

					if ( addParam )
					{
						args = "#args#, #kName#='#kVal#'";
						argList = "#argList#,#kName#";
					}
				}
				if (kName eq 'Src_url')
				{
					args = "#args#, src='#kVal#'";
					argList = "#argList#,src";
				}
			}

			// For YouTube pass in the protocol and the host as args and part of the argsList
			if (rInfo.serviceID == Request.Constants.factoryMultimediaService.YOUTUBE_ID)
			{
				args = "#args#, protocol='#protocol#'";
				argList = "#argList#,protocol";

				args = "#args#, host='#host#'";
				argList = "#argList#,host";
			}

			linkArgs['fname'] = fn;
			linkArgs['obj'] = args;
			linkArgs['argList'] = argList;

			return linkArgs;
		</cfscript>

	</cffunction>

</cfcomponent>